<?php

// declare(strict_types=1);

namespace W3ctrl\Fsfpay\Controller\Fsfpay;

use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\ResponseInterface;
use Magento\Framework\Controller\ResultInterface;
use Magento\Framework\View\Result\Page;
use Magento\Framework\View\Result\PageFactory;
use W3ctrl\Fsfpay\Helper\Data;
/**
 * Class Index
 */
class Create extends Action
{
    /**
     * @var PageFactory
     */
    private $pageFactory;
    private $jsonResultFactory;
    protected $orderRepository;
    protected $quoteRepository;
    private $helper;
    /**
     * @param Context $context
     * @param PageFactory $pageFactory
     */
    public function __construct(
        Context $context,
        PageFactory $pageFactory,
        \Magento\Framework\Controller\Result\JsonFactory $jsonResultFactory,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Magento\Quote\Api\CartRepositoryInterface $quoteRepository,
        \Magento\Sales\Model\OrderFactory $orderFactory,
        \Magento\Checkout\Model\Session $checkoutSession,
        Data $helper
        // \Psr\Log\LoggerInterface $logger
    ) {
        parent::__construct($context);
        $this->_orderFactory = $orderFactory;
        $this->checkoutSession = $checkoutSession;
        $this->pageFactory = $pageFactory;
        $this->jsonResultFactory = $jsonResultFactory;
        $this->orderRepository = $orderRepository;
        $this->quoteRepository = $quoteRepository;
        $this->helper = $helper;
        // $this->_logger = $logger;
    }

    /**
     * @return ResponseInterface|ResultInterface|Page
     */
    public function execute()
    {
        $objectManager = \Magento\Framework\App\ObjectManager::getInstance();
        $storeManager = $objectManager->get('\Magento\Store\Model\StoreManagerInterface');
        $storeURL = $storeManager->getStore()->getBaseUrl();
        $currencySymbol = $storeManager->getStore()->getDefaultCurrencyCode();
        $result = $this->jsonResultFactory->create();
         if ($this->helper->isTestMode()) {
            $mode =1;
        } else {
           $mode=0;
        }

        $order = $this->getOrder();
        $orderId = $order->getIncrementId();
        $amount = $order->getGrandTotal();
        $billingAddress = $order->getBillingAddress();
        $email = $billingAddress->getEmail();

        $data = [
            'apiKey' => $this->helper->getApiKey(),
            'apiUsername' => $this->helper->getApiUsername(),
            'orderId' => $orderId,
            'email' => $email,
            'amount' => $amount,
            'currency' => $currencySymbol,
            'cryptocurrency' => 'BTC',
            'testmode' => $mode,
            'callbackUrl' => $storeURL.'fsfpay/fsfpay/response',
            'successRedirectUrl' => $storeURL.'fsfpay/fsfpay/success',
            'language' => 'en'
        ];
$response = $this->helper->simplePost($data);

        // Create W3ctrl
        if (!$order) {
            paytabs_error_log("Paytabs: Order is missing!, Quote = [{$quoteId}]");
            $result->setData([
                'result' => 'Order is missing!'
            ]);
            return $result;
        }

        $result->setData($response);

        return $result;
    }


    public function getOrder()
    {
        $lastRealOrderId = $this->checkoutSession->getLastRealOrderId();
        if ($lastRealOrderId) {
            $order = $this->_orderFactory->create()->loadByIncrementId($lastRealOrderId);
            return $order;
        }
        return false;
    }
}
